<?php

namespace jrosset\CliProgram\Requirements;

use jrosset\CliProgram\CliHelper;
use ReflectionClass;
use Symfony\Component\Console\ConsoleEvents;
use Symfony\Component\Console\Event\ConsoleCommandEvent;
use Symfony\Component\EventDispatcher\EventDispatcher;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;
use Throwable;

/**
 * An application with command requirements checking
 *
 * @see IRequirements
 */
trait TRequirementsApplication {
    /**
     * Register the listener for command requirements
     *
     * @param EventDispatcherInterface|null $dispatcher The event dispatcher is already existing
     *
     * @return void
     */
    protected final function registerCommandRequirementsListener (?EventDispatcherInterface $dispatcher = null): void {
        $dispatcher ??= new EventDispatcher();
        $dispatcher->addListener(ConsoleEvents::COMMAND, [$this, 'checkCommandRequirements']);
        $this->setDispatcher($dispatcher);
    }

    /**
     * Check the requirements of a command
     *
     * @param ConsoleCommandEvent $event The command event
     *
     * @return void
     */
    public function checkCommandRequirements (ConsoleCommandEvent $event): void {
        $commandInput = $event->getInput();
        $commandOutput = $event->getOutput();
        try {
            //region Check the command is valid
            if (($command = $event->getCommand()) === null) {
                return;
            }

            $commandReflection = new ReflectionClass($command);
            //endregion
            //region Contrôle pré-requis (implémentation directe)
            if ($command instanceof IRequirements) {
                $command->checkRequirements($commandInput, $commandOutput);
            }
            //endregion
        }
        catch (Throwable $exception) {
            $this->renderThrowable($exception, CliHelper::getErrorOutput($commandOutput));

            $event->disableCommand();
            $event->stopPropagation();
            return;
        }
    }
}